﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using AIOUSBNet;  // the namespace exposes the AIOUSB Class interface 

namespace Sample1
{
    public partial class Form1 : Form
    {
        public System.Threading.Thread wrker;
        double BaseHz = 0.1;
        double Hz;
        bool bEnabledReference;
        uint DataSize = 8;
        const int DAC_CHANNEL_COUNT = 16;

        private void BuildWave(out ushort[] Wave, double S, double E, double R, double D)
        {
            int I, L, K;
            double Completion, F, F0, dF, T,M,B;
            L = (int)(R * D); // "D" seconds worth
            L = L * DAC_CHANNEL_COUNT; // n channels worth

            Wave = new ushort[L];

            //  SetLength(Result, L);
            F0 = 2 * Math.PI * S / R;
            dF = 2 * Math.PI * (E - S) / R;

            for (T = 0, I = 0; I < L; )
            {
                Completion = (double)I / (double)L;
                F = S + Completion * dF;
                T = T + F;

                ushort Sample=0;

                if (rbChirpWave.Checked)
                   Sample = (ushort)(0x7FFF + 0x7FFF * Math.Sin(T) * (1.0 - Math.Cos(2.0 * Math.PI * Completion)) / 2.0);
                else if (rbSquareWave.Checked)
                   Sample = (ushort)(((I >> 3) & 1) * 0xFFFF); // little square wave toggles

                M = 1; B = 0;
                if (cbDivisions.Checked) 
                {
                    M = 1.0 / DAC_CHANNEL_COUNT; B = M * (2^16);
                }

                for (K = 0; K < DAC_CHANNEL_COUNT; K++)
                    Wave[I++] = (ushort)(Sample * M + K * B);
            }
        }

        private void displayDebugC7(byte[] buf)
        {
            label3.Text = "C7 Read: " + DataSize + " bytes\r\n";
            label3.Text += Convert.ToString(buf[0], 16).PadLeft(2, '0') + " length\r\n";
            label3.Text += Convert.ToString(buf[2], 16).PadLeft(2, '0') + Convert.ToString(buf[1], 16).PadLeft(2, '0') + " Timer Divisor\r\n";

            label3.Text += Convert.ToString(buf[3], 16).PadLeft(2, '0') + " DACChan\r\n";
            label3.Text += Convert.ToString(buf[4], 16).PadLeft(2, '0') + " WAVES_ENABLED\r\n";
            label3.Text += Convert.ToString(buf[6], 10) + Convert.ToString(buf[5], 16).PadLeft(2, '0') + " myBufCount\r\n";
        }

        private void displayDebugC7Read()
        {
            byte[] buf = new byte[DataSize];
            AIOUSB.GenericVendorRead(AIOUSB.diOnly, 0xC7, 0, 0x0000, ref DataSize, buf);
            displayDebugC7(buf);
        }
        
        private bool HandleStatus(uint Status, string msg)
        {
            bool result = true;
            if (Status != 0)
            {
                label2.Text += "\n" + msg + (Status);
                if ((Status == 2) || (Status == 1617))
                {
                    label2.Text += " Attempting redetect";
                    AIOUSB.GetDevices();
                }
                result = false;
            }
            // Status = AIOUSB.GenericVendorRead(AIOUSB.diOnly, 0xC7, 0, 0, ref DataSize, buf);
            // displayDebugC7Read(buf, DataSize);
            return result;
        }

        public Form1()
        {
            InitializeComponent();
            bEnabledReference = false;
            if (AIOUSB.GetDevices() == 0)
            {
                label2.Text+= " - no card found - "; 
            }
        }

        private double TrackPosToFreq( int pos )
        {
            return BaseHz * Math.Pow(2, pos/12.0);
        }
                
        private void trackBarFreqX_Scroll(object sender, EventArgs e)
        {
            lblStartF.Text = ((UInt32)(10.0 * TrackPosToFreq(trackBar0.Value)) / 10.0).ToString() + " Hz";
        }

        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            lblEndF.Text = ((UInt32)(10.0 * TrackPosToFreq(trackBar1.Value)) / 10.0).ToString() + " Hz";
        }

        private void trackBar2_Scroll(object sender, EventArgs e)
        {
            lblSampleRate.Text = trackBar2.Value + " Hz";
        }

        private void btnPlay_Click(object sender, EventArgs e)
        {
            btnPlay.Enabled = false;
            if (btnStop.Enabled) btnStop_Click(sender, e);

            butReset_Click(sender, e);
            butEnableWave_Click(sender, e);

            if (!bEnabledReference)
            {
                HandleStatus(AIOUSB.DACSetBoardRange(AIOUSB.diOnly, 0), "DACSetBoardRange:");
                bEnabledReference = true;
            }

            double S, E, R, D;
            ushort[] Wave;
            S = TrackPosToFreq(trackBar0.Value);
            E = TrackPosToFreq(trackBar1.Value);
            R = trackBar2.Value;
            D = trackBar3.Value;

            label3.Text += "Blocking during output...\r\n";
            Application.DoEvents();
            BuildWave(out Wave, S, E, R, D);



            HandleStatus(AIOUSB.DACOutputProcess(AIOUSB.diOnly, ref R, (uint)Wave.Length, Wave), "DACOutputProcess:");
            label2.Text += "Len: " + Wave.Length + " words\r\n";
            //displayDebugC7Read();
            label3.Text += "...unblocked\r\n";
            label3.Text += "Buffer draining:\r\n";

            do { 
                System.Threading.Thread.Sleep(10); 
            } while (GetDACSlowStreamStatus());

            label3.Text += "Output Complete:\r\n";

            btnPlay.Enabled = true;

            //GC.Collect(GC.MaxGeneration, GCCollectionMode.Forced); // DON'T DO THIS!!!
        }

        private bool GetDACSlowStreamStatus()
        {
            byte[] buf = new byte[DataSize];
            AIOUSB.GenericVendorRead(AIOUSB.diOnly, 0xC7, 0, 0x0000, ref DataSize, buf);
            int remainder = ((buf[6] << 8) | buf[5]);
            label3.Text += Convert.ToString(remainder,16) + "\r\n";
            if ( remainder > 0) return true;
            return false;
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            wrker.Abort();
            do { System.Threading.Thread.Sleep(1); } while (wrker.IsAlive);
            AIOUSBNet.AIOUSB.DACOutputClose(AIOUSB.diOnly, 0);
            btnLoop.Enabled = true;
            btnStop.Enabled = false;
            btnPlay.Enabled = true;
        }

        private void btnLoop_Click(object sender, EventArgs e)
        {
            if (btnStop.Enabled) btnStop_Click(sender, e);
            btnStop.Enabled = false;
            btnLoop.Enabled = true;
            btnPlay.Enabled = false;


            if (!bEnabledReference)
            {
                HandleStatus(AIOUSB.DACSetBoardRange(AIOUSB.diOnly, 0), "DACSetBoardRange:");
                bEnabledReference = true;
            }

            double S, E, R;
            ushort[] Wave;
            S = TrackPosToFreq(trackBar0.Value);
            E = TrackPosToFreq(trackBar1.Value);
            Hz = R = trackBar2.Value;

            BuildWave(out Wave, S, E, R,1);

            wrker = new System.Threading.Thread(this.WorkerThread);
            AIOUSB.DACOutputOpen(AIOUSB.diOnly, ref Hz);
            wrker.Start(Wave);

            btnStop.Enabled = true;
            btnLoop.Enabled = false;
        }

        public void WorkerThread(object StreamWave)
        {

            do
              AIOUSB.DACOutputFrameRaw(AIOUSB.diOnly, (uint)(((ushort[]) StreamWave).Length), (ushort[])StreamWave);
            while (wrker.ThreadState == System.Threading.ThreadState.Running);

        }

        private void button1_Click(object sender, EventArgs e)
        {
            HandleStatus(AIOUSB.DACDirect(AIOUSB.diOnly, 0, 0x1000), "DACDirect:"); // CH 0 only
            displayDebugC7Read();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            HandleStatus(AIOUSB.DACDirect(AIOUSB.diOnly, 0, 0xE000), "DACDirect:");
            displayDebugC7Read();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            HandleStatus(AIOUSB.DACSetBoardRange(AIOUSB.diOnly, 0), "DACSetBoardRange:");
            displayDebugC7Read();
            bEnabledReference = true;
        }


        private void butReset_Click(object sender, EventArgs e)
        {
            byte[] buf = new byte[DataSize];
            label2.Text = "";
            label3.Text = "";
            HandleStatus(AIOUSB.GenericVendorRead(AIOUSB.diOnly, 0xC7, 0, 0x8000, ref DataSize, buf),"Reset");
            displayDebugC7(buf);
            Application.DoEvents();
        }

        private void butPop_Click(object sender, EventArgs e)
        {
            byte[] buf = new byte[DataSize];
            ushort counts;

            HandleStatus(AIOUSB.GenericVendorRead(AIOUSB.diOnly, 0xC7, 0, 0x0002, ref DataSize, buf), "Pop");
            displayDebugC7(buf);

            counts = (ushort)(buf[6] + (buf[7] << 8));
            AIOUSB.DACDirect(AIOUSB.diOnly, 0, counts); // --J2H Debug: act as if the firmware was playing the waveform one datapoint per tick.
        }

        private void butPush_Click(object sender, EventArgs e)
        {
            byte[] buf = new byte[DataSize];
            HandleStatus(AIOUSB.GenericVendorRead(AIOUSB.diOnly, 0xC7, 0, 0x0001, ref DataSize, buf),"Push");
            displayDebugC7(buf);
        }

        private void butDisableWave_Click(object sender, EventArgs e)
        {
            byte[] buf = new byte[DataSize];
            HandleStatus(AIOUSB.GenericVendorWrite(AIOUSB.diOnly, 0xC7, 0, 0,  0, buf),"Disable");
            displayDebugC7Read();
        }

        private void butEnableWave_Click(object sender, EventArgs e)
        {
            byte[] buf = new byte[DataSize];
            HandleStatus(AIOUSB.GenericVendorWrite(AIOUSB.diOnly, 0xC7, 1, 0, 0, buf),"Enable");
            displayDebugC7Read(); // note: this debug statement doesn't display the enable state correctly
            
        }

        private void butReadDebugC7_Click(object sender, EventArgs e)
        {
            displayDebugC7Read();
        }

        private void trackBar3_Scroll(object sender, EventArgs e)
        {
            lblDuration.Text = trackBar3.Value + " Seconds";
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            trackBarFreqX_Scroll(sender, e);
            trackBar1_Scroll(sender, e);
            trackBar2_Scroll(sender, e);
            trackBar3_Scroll(sender, e);
            butReadDebugC7_Click(sender, e);
        }
    }
}